@@ -159,6 +159,30 @@ def update_run(values, run_id, session=None):
     return run
 
 
+def update_test_run(values, test_run_id, session=None):
+    """Update an individual test_run with new data.
+
+    This method will take a dictionary of fields to update for a specific
+    test_run. If a field is omitted it will not be changed in the DB.
+
+    :param dict values: dict of values to update the test with. The key is the
+                        column name and the value is the new value to be stored
+                        in the DB
+    :param str test_run_id: the uuid of the test_run to update. (value of the
+                        id column for the row to be updated)
+    :param session: optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return: The updated test_run object stored in the DB
+    :rtype: subunit2sql.models.TestRun
+    """
+    session = session or get_session()
+    with session.begin():
+        test_run = get_test_run_by_id(test_run_id, session)
+        test_run.update(values)
+    return test_run
+
+
 def add_run_metadata(meta_dict, run_id, session=None):
     """Add a metadata key value pairs for a specific run.
 
@@ -211,8 +235,14 @@ def create_test_run(test_id, run_id, status, start_time=None,
     test_run.test_id = test_id
     test_run.run_id = run_id
     test_run.status = status
-    test_run.stop_time = end_time.replace(tzinfo=None)
-    test_run.start_time = start_time.replace(tzinfo=None)
+    start_time = start_time.replace(tzinfo=None)
+    start_time_microsecond = start_time.microsecond
+    stop_time = end_time.replace(tzinfo=None)
+    stop_time_microsecond = stop_time.microsecond
+    test_run.stop_time = stop_time
+    test_run.stop_time_microsecond = stop_time_microsecond
+    test_run.start_time = start_time
+    test_run.start_time_microsecond = start_time_microsecond
     session = session or get_session()
     with session.begin():
         session.add(test_run)
@@ -453,6 +483,11 @@ def get_test_run_duration(test_run_id, session=None):
     """
     session = session or get_session()
     test_run = get_test_run_by_id(test_run_id, session)
+    start_time = test_run.start_time
+    start_time = start_time.replace(
+        microsecond=test_run.start_time_microsecond)
+    stop_time = test_run.stop_time
+    stop_time = stop_time.replace(microsecond=test_run.stop_time_microsecond)
     return read_subunit.get_duration(test_run.start_time, test_run.stop_time)
 
 
@@ -494,23 +529,87 @@ def get_tests_run_dicts_from_run_id(run_id, session=None):
     session = session or get_session()
     query = db_utils.model_query(models.Test, session=session).join(
         models.TestRun).filter_by(run_id=run_id).join(
-            models.TestRunMetadata).values(models.Test.test_id,
-                                           models.TestRun.status,
-                                           models.TestRun.start_time,
-                                           models.TestRun.stop_time,
-                                           models.TestRunMetadata.key,
-                                           models.TestRunMetadata.value)
+            models.TestRunMetadata).values(
+                models.Test.test_id,
+                models.TestRun.status,
+                models.TestRun.start_time,
+                models.TestRun.start_time_microsecond,
+                models.TestRun.stop_time,
+                models.TestRun.stop_time_microsecond,
+                models.TestRunMetadata.key,
+                models.TestRunMetadata.value)
     test_runs = {}
     for test_run in query:
         if test_run[0] not in test_runs:
+            start_time = test_run[2]
+            start_time = start_time.replace(microsecond=test_run[3])
+            stop_time = test_run[4]
+            stop_time = stop_time.replace(microsecond=test_run[5])
             test_runs[test_run[0]] = {
                 'status': test_run[1],
-                'start_time': test_run[2],
-                'stop_time': test_run[3],
+                'start_time': start_time,
+                'stop_time': stop_time,
             }
-            if test_run[4]:
-                test_runs[test_run[0]]['metadata'] = {test_run[4]: test_run[5]}
+            if test_run[6]:
+                test_runs[test_run[0]]['metadata'] = {test_run[6]: test_run[7]}
         else:
-            if test_run[4]:
-                test_runs[test_run[0]]['metadata'][test_run[4]] = test_run[5]
+            if test_run[6]:
+                test_runs[test_run[0]]['metadata'][test_run[6]] = test_run[7]
     return test_runs
+
+
+def get_test_run_time_series(test_id, session=None):
+    """Returns a time series dict of run_times for successes of a single test
+
+    :param str test_id: The test's uuid (the id column in the test table) which
+                        will be used to get all the test run times for.
+    :param session: optional session object if one isn't provided a new session
+
+    :return dict: A dictionary with the start times as the keys and the values
+                  being the duration of the test that started at that time in
+                  sec.
+    """
+    session = session or get_session()
+    query = db_utils.model_query(models.TestRun, session=session).filter_by(
+        test_id=test_id).filter_by(status='success').values(
+            models.TestRun.start_time, models.TestRun.start_time_microsecond,
+            models.TestRun.stop_time, models.TestRun.stop_time_microsecond)
+    time_series = {}
+    for test_run in query:
+        start_time = test_run[0]
+        start_time = start_time.replace(microsecond=test_run[1])
+        stop_time = test_run[2]
+        stop_time = stop_time.replace(microsecond=test_run[3])
+        time_series[test_run[0]] = (stop_time - start_time).total_seconds()
+    return time_series
+
+
+def get_recent_successful_runs(num_runs=10, session=None):
+    """Return a list of run uuid strings for the most recent successful runs
+
+    :param int num_runs: The number of runs to return in the list
+    :param session: optional session object if one isn't provided a new session
+
+    :return list: A list of run uuid strings (the id column in the runs table)
+                  for the most recent runs.
+    """
+    session = session or get_session()
+    results = db_utils.model_query(models.Run, session).order_by(
+        models.Run.run_at.desc()).filter_by(fails=0).limit(num_runs).all()
+    return map(lambda x: x.id, results)
+
+
+def get_recent_failed_runs(num_runs=10, session=None):
+    """Return a list of run uuid strings for the most recent failed runs
+
+    :param int num_runs: The number of runs to return in the list
+    :param session: optional session object if one isn't provided a new session
+
+    :return list: A list of run uuid strings (the id column in the runs table)
+                  for the most recent runs.
+    """
+    session = session or get_session()
+    results = db_utils.model_query(models.Run, session).order_by(
+        models.Run.run_at.desc()).filter(
+        models.Run.fails > 0).limit(num_runs).all()
+    return map(lambda x: x.id, results)
